package HaloImpl;

import Halo.*;
import java.lang.*;
import java.util.*;

public class NyilvantartoImpl extends NyilvantartoPOA 
{
  private class LeltarElem
  {
    public Elem elem;
    public Dictionary kapcs;

    public LeltarElem( Elem elem_ )
    {
      elem = elem_;
      kapcs = new Hashtable();
    }
  };

  private Dictionary leltar = new Hashtable();
  private Random random = new Random();

  private String kovAzon() // general egy azonositot
  {
    int r = ( Math.abs( random.nextInt() ) % 100 );
    int k = 100 * r + leltar.size() + 1;
    return "a" + k;
  }

  public SzamGep ujSzamGep( String proc )
  {
    String azon = kovAzon(); // egyedi azonosito
    SzamGepImpl szgep = new SzamGepImpl( azon, proc );
    leltar.put( azon, new LeltarElem( szgep ) );
    return szgep;
  }

  public Nyomtato ujNyomtato( boolean szines )
  {
    String azon = kovAzon(); // egyedi azonosito
    NyomtatoImpl nyomt = new NyomtatoImpl( azon, szines );
    leltar.put( azon, new LeltarElem( nyomt ) );
    return nyomt;
  }

  public void osszeKapcs( String elAz1, String elAz2 ) 
    throws HibasAzonosito
  {
    LeltarElem lelElem1 = ( LeltarElem ) leltar.get( elAz1 );
    LeltarElem lelElem2 = ( LeltarElem ) leltar.get( elAz2 );

    if( lelElem1 == null || lelElem2 == null )
    {
      throw new HibasAzonosito(); // valamelyik azonosito rossz
    }

    Dictionary kapcs1 = lelElem1.kapcs;
    Dictionary kapcs2 = lelElem2.kapcs;

    kapcs1.put( elAz2, lelElem2.elem );
    kapcs2.put( elAz1, lelElem1.elem );
  }

  public void szetBont( String elAz1, String elAz2 ) 
    throws HibasAzonosito
  {
    LeltarElem lelElem1 = ( LeltarElem ) leltar.get( elAz1 );
    LeltarElem lelElem2 = ( LeltarElem ) leltar.get( elAz2 );

    if( lelElem1 == null || lelElem2 == null )
    {
      throw new HibasAzonosito(); // valamelyik azonosito rossz
    }

    Dictionary kapcs1 = lelElem1.kapcs;
    Dictionary kapcs2 = lelElem2.kapcs;

    kapcs1.remove( elAz2 );
    kapcs2.remove( elAz1 );
  }

  public Elem torol( String elAz ) // torol egy elemet
    throws HibasAzonosito
  {
    LeltarElem lelElem = ( LeltarElem ) leltar.get( elAz );

    if( lelElem == null ) 
    {
      throw new HibasAzonosito(); // az azonosito rossz
    }

    {
      int i = 0; 
      Enumeration e = lelElem.kapcs.keys(); 
      for( ; e.hasMoreElements(); ++i ) 
      {
        String elAz2 = ( String ) e.nextElement();
        LeltarElem lelElem2 = ( LeltarElem ) leltar.get( elAz2 );

        if( lelElem2 == null )
        {
          throw new HibasAzonosito(); // hibas azonosito
        }

        Dictionary kapcs2 = lelElem2.kapcs;
        kapcs2.remove( elAz );
      }
    }

    leltar.remove( elAz );

    return lelElem.elem;
  }

  public Elem leker( String elAz ) // leker egy elemet
    throws HibasAzonosito
  {
    LeltarElem lelElem = ( LeltarElem ) leltar.get( elAz );

    if( lelElem == null ) 
    {
      throw new HibasAzonosito(); // az azonosito rossz
    }

    return lelElem.elem;
  }

  public ElemLista lista()
  {
    ElemLista elLista = new ElemListaImpl( leltar.size() );

    {
      int i = 0; 
      Enumeration e = leltar.elements(); 
      for( ; e.hasMoreElements(); ++i ) 
      {
        LeltarElem lelElem = ( LeltarElem ) e.nextElement();
        elLista.elemek[ i ] = lelElem.elem;
      }
    }

    return elLista;
  }

  public ElemLista kapcs( String elAz ) 
    throws HibasAzonosito
  {
    LeltarElem lelElem = ( LeltarElem ) leltar.get( elAz );

    if( lelElem == null )
    {
      throw new HibasAzonosito(); // az azonosito rossz
    }

    ElemLista elLista = new ElemListaImpl( lelElem.kapcs.size() );

    {
      int i = 0; 
      Enumeration e = lelElem.kapcs.elements(); 
      for( ; e.hasMoreElements(); ++i ) 
      {
        elLista.elemek[ i ] = ( Elem ) e.nextElement();
      }
    }

    return elLista;
  }

  public boolean vanKapcs( String elAz1, String elAz2 ) 
    throws HibasAzonosito, HibasKapcsolat
  {
    LeltarElem lelElem1 = ( LeltarElem ) leltar.get( elAz1 );
    LeltarElem lelElem2 = ( LeltarElem ) leltar.get( elAz2 );

    if( lelElem1 == null || lelElem2 == null )
    {
      throw new HibasAzonosito(); // valamelyik azonosito rossz
    }

    Dictionary kapcs1 = lelElem1.kapcs;
    Dictionary kapcs2 = lelElem2.kapcs;

    boolean vanKapcs12 = ( kapcs1.get( elAz2 ) != null );
    boolean vanKapcs21 = ( kapcs2.get( elAz1 ) != null );

    if( 
        ( vanKapcs12 && !vanKapcs21 ) || 
        ( !vanKapcs12 && vanKapcs21 ) 
      )
    {
      throw new HibasKapcsolat(); // csak egyik irany van meg
    }

    return vanKapcs12 && vanKapcs21;
  }
};
